unit Ship;

{$MODE Delphi}

interface

uses
	Classes, Math2D;

type
	TPolygonLinesArr = array of TLine2D;

  TPolygonObj = class(TPersistent)
  private
  	fPos: TPointF;
    fAngle, fScale: Double;
    fPolygons, fDimensions: TPolygonLinesArr;
    procedure Calculate;
    procedure Rotate(Angle: Double);
		procedure SetPos(Value: TPointF);
		procedure SetAngle(Value: Double);
		procedure SetScale(Value: Double);
		procedure SetDimensions(Value: TPolygonLinesArr);
	public
		constructor Create(APos: TPointF; AAngle, AScale: Double;
			APolygons: TPolygonLinesArr); overload;
		property Pos: TPointF read fPos write SetPos;
		property Angle: Double read fAngle write SetAngle;
		property Scale: Double read fScale write SetScale;
		property Polygon: TPolygonLinesArr read fPolygons;
		property Dimensions: TPolygonLinesArr read fDimensions write SetDimensions;
	end;

	TShip = class(TPolygonObj)
	private
		fFirerate, fID: Integer;
	public
		constructor Create(APos: TPointF; AAngle, AScale: Double;
			APolygons: TPolygonLinesArr; AFirerate, AID: Integer); overload;
	published
		property Firerate: Integer read fFirerate write fFirerate;
		property ID: Integer read fID write fID;
	end;

  TAsteroid = class(TPolygonObj)

  end;

implementation

procedure TPolygonObj.Calculate;
var
	I: Integer;
begin
	for I := Low(fPolygons) to High(fPolygons) do begin
		fPolygons[I].x1 := fPos.x + fDimensions[I].x1 * fScale;
    fPolygons[I].y1 := fPos.y + fDimensions[I].y1 * fScale;
    fPolygons[I].x2 := fPos.x + fDimensions[I].x2 * fScale;
    fPolygons[I].y2 := fPos.y + fDimensions[I].y2 * fScale;
  end;
end;

procedure TPolygonObj.Rotate(Angle: Double);
var
	I: Integer;
begin
	for I := Low(fPolygons) to High(fPolygons) do begin
  	RotatePointFA(fPolygons[I].x1, fPolygons[I].y1, fPos.x, fPos.y, Angle);
    RotatePointFA(fPolygons[I].x2, fPolygons[I].y2, fPos.x, fPos.y, Angle);
  end;
end;

procedure TPolygonObj.SetPos(Value: TPointF);
begin
	if not PointsEqualF(fPos, Value) then
	begin
		fPos := Value;
		Calculate;
    Rotate(fAngle);
	end;
end;

procedure TPolygonObj.SetAngle(Value: Double);
begin
	if fAngle <> Value then
	begin
  	Rotate(Value - fAngle);
		fAngle := Value;
		if fAngle > 360 then
			fAngle := fAngle - 360
		else if fAngle < 0 then
			fAngle := fAngle + 360;
	end;
end;

procedure TPolygonObj.SetScale(Value: Double);
begin
	if fScale <> Value then
	begin
		fScale := Value;
		Calculate;
    Rotate(fAngle);
	end;
end;

procedure TPolygonObj.SetDimensions(Value: TPolygonLinesArr);
begin
	fDimensions := Copy(Value, Low(Value), Length(Value));
	fPolygons := Copy(Value, Low(Value), Length(Value));
	Calculate;
  Rotate(fAngle);
end;

constructor TPolygonObj.Create(APos: TPointF; AAngle, AScale: Double;
	APolygons: TPolygonLinesArr);
begin
	inherited Create;
	fPos := APos;
	fAngle := AAngle;
	fScale := AScale;
	fPolygons := APolygons;
	Calculate;
  Rotate(fAngle);
end;

constructor TShip.Create(APos: TPointF; AAngle, AScale: Double;
	APolygons: TPolygonLinesArr; AFirerate, AID: Integer);
begin
	inherited Create;
	fPos := APos;
	fAngle := AAngle;
	fScale := AScale;
	fPolygons := APolygons;
	fFirerate := AFirerate;
	fID := AID;
	Calculate;
  Rotate(fAngle);
end;

end.
